/*
 * Contributor(s): Natasha Noy noy@smi.stanford.edu
 */
package org.bioontology.bioportal.metadata;

import java.net.*;
import java.util.*;

import edu.stanford.smi.protegex.owl.model.*;

public class OntologyMetadata {

	public static Collection<OWLIndividual> getAllVirtualOntologies () {
		return Constants.virtualOntologyClass.getInstances(true);
	}
	
	/**
	 * @param   virtualOntologyURI  a virtual URI for the ontology
	 */
	
	public static OWLIndividual getCurrentVersion (URI virtualOntologyURI) throws Exception  {
		OWLIndividual virtualOntologyInstance = Util.getVirtualOntologyFromURI (virtualOntologyURI);
		return getCurrentVersion (virtualOntologyInstance);
	}

	/**
	 * @param   virtualOntologyIndividual  an OWL individual representig a virtual ontology
	 */
	
	public static OWLIndividual getCurrentVersion (OWLIndividual virtualOntologyInstance)  {
		return (OWLIndividual) virtualOntologyInstance.getPropertyValue(Constants.currentVersionProperty);
	}
	
	
	/**
	 * @param    virtualOntologyURI  a virtual URI for the ontology
	 */
	
	
	public static Collection<PropertyValue> getCoreMetadataForCurrentVersion (URI virtualOntologyURI) {
		OWLIndividual virtualOntologyInstance = Util.getVirtualOntologyFromURI (virtualOntologyURI);
		return getCoreMetadataForCurrentVersion (virtualOntologyInstance);
	}

	/**
	 * @param    virtualOntologyIndividual  an OWL individual representing a virtual ontology
	 */
	
	
	public static Collection<PropertyValue> getCoreMetadataForCurrentVersion (OWLIndividual virtualOntologyInstance) {
		OWLIndividual currenVerisonInstance = (OWLIndividual) virtualOntologyInstance.getPropertyValue(Constants.currentVersionProperty);
		
		Collection<PropertyValue> result = new ArrayList<PropertyValue> ();
		for (Iterator i = Constants.coreProperties.iterator(); i.hasNext(); ) {
			String nextCoreProperty = (String)i.next();
			PropertyValue nextPair = new PropertyValue ();
			nextPair.property = nextCoreProperty;
			nextPair.value = currenVerisonInstance.getPropertyValue(Constants.bioportalMetadataKnowledgeBase.getRDFProperty(nextCoreProperty)).toString();
			result.add(nextPair);
		}
		return result;
		
	}

	/**
	 * @param    virtualOntologyIndividual  an OWL individual representig a virtual ontology
	 */
	
	public static Collection<OWLIndividual> getOntologyVersions (OWLIndividual virtualOntologyInstance) {
		return virtualOntologyInstance.getPropertyValues(Constants.versionsProperty);
	}

	/**
	 * @param    virtualOntologyIndividual  an OWL individual representig a virtual ontology
	 */
	
	public static Collection<OWLIndividual> getOntologyViews (OWLIndividual virtualOntologyInstance) {
		return virtualOntologyInstance.getPropertyValues(Constants.viewsProperty);
	}

	/**
	 * @param    virtualOntologyIndividual  an OWL individual representig a virtual ontology
	 * @param    property  an RDF Property
	 */
	
	public static Collection getPropertyValues (OWLIndividual virtualOntologyInstance, RDFProperty property) {
		return virtualOntologyInstance.getPropertyValues(property);
	}
	
	/**
	 * @param    virtualOntologyURI  virtual URI for the ontology
	 * @param		currentVerisonURI 	  the URI for the current version
	 * @param    pairs  property-value pairs for the metadata properties 
	 */
	
	public static OWLIndividual createNewVersion (URI virtualOntologyURI, URI currentVersionURI, Collection<PropertyValue> pairs) {
		OWLIndividual virtualOntologyInstance = Util.getVirtualOntologyFromURI (virtualOntologyURI);
		OWLIndividual versionInstance = (OWLIndividual) Constants.omvOntologyClass.createInstance (currentVersionURI.toString());
		
		for (Iterator i = pairs.iterator(); i.hasNext(); ) {
			PropertyValue nextPair = (PropertyValue) i.next();
			RDFProperty property = Constants.bioportalMetadataKnowledgeBase.getRDFProperty(nextPair.property); 
			versionInstance.setPropertyValue(property, nextPair.value);
		}
		
		virtualOntologyInstance.addPropertyValue(Constants.versionsProperty, versionInstance);
		
		return versionInstance;
		
	}

	
	
}
