/*
 * Contributor(s): Natasha Noy noy@smi.stanford.edu
 */
package org.bioontology.bioportal.metadata;

import java.net.*;
import java.util.*;

import edu.stanford.smi.protege.util.*;
import edu.stanford.smi.protegex.owl.model.*;

public class Reviews {
	/**
	 * @param    virtualOntologyIndividual  an OWL individual representing a virtual ontology
	 */
	
	
	public static Collection<OWLIndividual> getReviewsForAnOntology (OWLIndividual virtualOntologyInstance) {
		String virtualOntologyURI = (String)virtualOntologyInstance.getPropertyValue(Constants.virtualURIProperty);
		return getReviewsForAnOntology (virtualOntologyURI);		
	}

	
	
	/**
	 * @param    virtualOntologyURI  a virtual URI for the ontology
	 */
	
	
	public static Collection<OWLIndividual> getReviewsForAnOntology (String virtualOntologyURI) {
		Collection<OWLIndividual> relevantAnnotations = Constants.bioportalMetadataKnowledgeBase.getFramesWithValue(Constants.annotatesByVirtualIdProperty, null, false, virtualOntologyURI);
		return relevantAnnotations;
	}

	
	/**
	 * @param    virtualOntologyURI  a virtual URI for the ontology
	 */
	
	
	public static Collection<OWLIndividual> getReviewsForCurrentVersion (URI virtualOntologyURI) {
		OWLIndividual virtualOntologyInstance = Util.getVirtualOntologyFromURI (virtualOntologyURI);
		return getReviewsForCurrentVersion (virtualOntologyInstance);
	}
	
	/**
	 * @param    virtualOntologyIndividual  an OWL individual representing a virtual ontology
	 */
	
	
	public static Collection<OWLIndividual> getReviewsForCurrentVersion (OWLIndividual virtualOntologyIndividual) {
		OWLIndividual currentVersionInstance = (OWLIndividual) virtualOntologyIndividual.getPropertyValue(Constants.currentVersionProperty);
		Collection<OWLIndividual> relatedAnnotations = currentVersionInstance.getPropertyValues(Constants.associatedAnnotationsProperty);

		return relatedAnnotations;		
	}

	/**
	 * @param    ontology  an OWL individual representing a version of an ontology
	 */
	
	
	public static Collection<OWLIndividual> getReviewsForVersion (OWLIndividual ontology) {
		return ontology.getPropertyValues(Constants.associatedAnnotationsProperty);		
	}

	/**
	 * @param    currentOntologyIndividual  an OWL individual representing a virtual ontology
	 * @param	project is an optional reference to the project if the review is performed in the context of a project
	 * @param	reviewElements is a collection of the review rating and textual values
	 */
	
	
	public static OWLIndividual createReview (OWLIndividual currentOntologyIndividual, OWLIndividual project, Collection<EvaluationDimensionData> reviewElements) {
		OWLIndividual review = (OWLIndividual)Constants.reviewClass.createInstance(null);
		
		review.setPropertyValue(Constants.annotatesProperty, currentOntologyIndividual);
		
		String virtualID = (String) currentOntologyIndividual.getPropertyValue(Constants.virtualURIProperty);
		review.setPropertyValue(Constants.annotatesByVirtualIdProperty, virtualID);
		
		for (Iterator i = reviewElements.iterator(); i.hasNext(); ) {
			EvaluationDimensionData next = (EvaluationDimensionData)i.next();
			OWLIndividual nextElement = (OWLIndividual)Constants.evaluationDimensionClass.createInstance(null);
			if (next.rating != null)
				nextElement.setPropertyValue(Constants.numericRatingProperty, next.rating.intValue());
			if (next.text != null)
				nextElement.setPropertyValue(Constants.textualReviewProperty, next.text);
			review.setPropertyValue(next.dimension, nextElement);
		}
		
		return review;
		
	}

	/**
	 * this method is an example of a method that takes a review instance and gets the numeric ratings to display from it:
	 * <domain of coverage, 1>
	 * This method is intended only as an example of how to disassenble a Review instance
	 * 
	 * @param review a review instance
	 * @return a set of property-value pairs for each of the dimensions, in the order in which they should be displayed
	 */
	public static ArrayList<PropertyValue> getReviewData (OWLIndividual review) {
		ArrayList<PropertyValue> result = new ArrayList<PropertyValue> ();
		for (Iterator i = Constants.reviewDimensions.iterator(); i.hasNext();) {
			RDFProperty nextDimension = (RDFProperty) i.next();
			OWLIndividual nextReviewElement = (OWLIndividual)review.getPropertyValue(nextDimension);
			if (nextReviewElement != null) {
				Integer nextRating = (Integer)nextReviewElement.getPropertyValue(Constants.numericRatingProperty);
				if (nextRating != null) {
					PropertyValue nextRecordElement = new PropertyValue ();		
					nextRecordElement.property = (String)CollectionUtilities.getFirstItem (nextDimension.getLabels());
					nextRecordElement.value = nextRating;
					result.add(result.size(), nextRecordElement);
				}
			}
		}
		return result;
	}

}
